@@ -43,8 +43,8 @@ line_push <- function(line, text,
   c(line, text)
 }
 
-spaces <- function(n) {
-  paste(rep(" ", n), collapse = "")
+spaces <- function(ns) {
+  map_chr(ns, function(n) paste(rep(" ", n), collapse = ""))
 }
 is_spaces <- function(str) {
   identical(str, spaces(nchar(str)))
@@ -203,7 +203,7 @@ fmls_deparse <- function(x, lines = new_lines()) {
   lines$increase_indent()
 
   while (!is_null(x)) {
-    lines$push(as_string(node_tag(x)))
+    sym_deparse(node_tag(x), lines)
 
     car <- node_car(x)
     if (!is_missing(car)) {
@@ -322,6 +322,11 @@ operand_deparse <- function(x, parent, side, lines) {
 }
 
 binary_op_deparse <- function(x, lines = new_lines(), space = " ") {
+  # Constructed call without second argument
+  if (is_null(node_cddr(x))) {
+    return(call_deparse(x, lines))
+  }
+
   outer <- x;
   op <- as_string(node_car(x))
 
@@ -352,26 +357,16 @@ unary_op_deparse <- function(x, lines = new_lines()) {
 brackets_deparse <- function(x, lines = new_lines()) {
   x <- node_cdr(x)
   lines$deparse(node_car(x))
-  lines$push_sticky("[")
-  lines$increase_indent()
 
-  x <- node_cdr(x)
-  lines$deparse(node_car(x))
-  lines$push_sticky("]")
-  lines$decrease_indent()
+  args_deparse(node_cdr(x), lines, delims = c("[", "]"))
 
   lines$get_lines()
 }
 brackets2_deparse <- function(x, lines = new_lines()) {
   x <- node_cdr(x)
   lines$deparse(node_car(x))
-  lines$push_sticky("[[")
-  lines$increase_indent()
 
-  x <- node_cdr(x)
-  lines$deparse(node_car(x))
-  lines$push_sticky("]]")
-  lines$decrease_indent()
+  args_deparse(node_cdr(x), lines, delims = c("[[", "]]"))
 
   lines$get_lines()
 }
@@ -388,6 +383,13 @@ braces_deparse <- function(x, lines = new_lines()) {
   lines$increase_indent()
 
   x <- node_cdr(x)
+
+  # No need for a newline if the block is empty
+  if (is_null(x)) {
+    lines$push(" }")
+    return(lines$get_lines())
+  }
+
   while (!is_null(x)) {
     lines$push_newline()
     lines$deparse(node_car(x))
@@ -402,11 +404,19 @@ braces_deparse <- function(x, lines = new_lines()) {
 }
 
 sym_deparse <- function(x, lines = new_lines()) {
-  lines$push(as_string(x))$get_lines()
+  str <- as_string(x)
+
+  if (needs_backticks(str)) {
+    str <- sprintf("`%s`", str)
+  }
+
+  lines$push(str)$get_lines()
 }
 
-args_deparse <- function(x, lines = new_lines()) {
-  lines$push_sticky("(")
+args_deparse <- function(x, lines = new_lines(), delims = c("(", ")")) {
+  stopifnot(is_character(delims, n = 2))
+
+  lines$push_sticky(delims[[1]])
   lines$increase_indent()
 
   while (!is_null(x)) {
@@ -424,7 +434,7 @@ args_deparse <- function(x, lines = new_lines()) {
     }
   }
 
-  lines$push_sticky(")")
+  lines$push_sticky(delims[[2]])
   lines$decrease_indent()
 
   lines$get_lines()
@@ -439,9 +449,7 @@ call_deparse <- function(x, lines = new_lines()) {
       lines$deparse(car)
     },
     backticks = {
-      lines$push_sticky("`")
       lines$deparse(node_car(car))
-      lines$push_sticky("`")
       args_deparse(node_cdr(car), lines)
     },
     lines$deparse(car)
@@ -696,8 +704,7 @@ default_deparse <- function(x, lines = new_lines()) {
 
 sexp_deparse <- function(x, lines = new_lines()) {
   if (is.object(x)) {
-    s3_deparse(x, lines)
-    return(NULL)
+    return(s3_deparse(x, lines))
   }
 
   deparser <- switch (typeof(x),
@@ -754,7 +761,7 @@ needs_backticks <- function(str) {
     return(TRUE)
   }
 
-  !grepl("^[[:alnum:]_.]+$", remaining)
+  grepl("[^[:alnum:]_.]", remaining)
 }
 
 # From gram.y
@@ -779,3 +786,139 @@ reserved_words <- c(
   "next",
   "break"
 )
+
+#' Create a default name for an R object
+#'
+#' @description
+#'
+#' `as_label()` transforms R objects into a short, human-readable
+#' description. You can use labels to:
+#'
+#' * Display an object in a concise way, for example to labellise axes
+#'   in a graphical plot.
+#'
+#' * Give default names to columns in a data frame. In this case,
+#'   labelling is the first step before name repair.
+#'
+#' See also [as_name()] for transforming symbols back to a
+#' string. Unlike `as_label()`, `as_string()` is a well defined
+#' operation that guarantees the roundtrip symbol -> string ->
+#' symbol.
+#'
+#' In general, if you don't know for sure what kind of object you're
+#' dealing with (a call, a symbol, an unquoted constant), use
+#' `as_label()` and make no assumption about the resulting string. If
+#' you know you have a symbol and need the name of the object it
+#' refers to, use [as_string()]. For instance, use `as_label()` with
+#' objects captured with `enquo()` and `as_string()` with symbols
+#' captured with `ensym()`.
+#'
+#' @param x An object.
+#'
+#' @section Transformation to string:
+#'
+#' * Quosures are [squashed][quo_squash] before being labelled.
+#' * Symbols are transformed to string with `as_string()`.
+#' * Calls are abbreviated.
+#' * Numbers are represented as such.
+#' * Other constants are represented by their type, such as `<dbl>`
+#'   or `<data.frame>`.
+#'
+#' Note that simple symbols should generally be transformed to strings
+#' with [as_name()]. Labelling is not a well defined operation and
+#' no assumption should be made about how the label is created. On the
+#' other hand, `as_name()` only works with symbols and is a well
+#' defined, deterministic operation.
+#'
+#' @seealso [as_name()] for transforming symbols back to a string
+#'   deterministically.
+#' @examples
+#' # as_label() is useful with quoted expressions:
+#' as_label(expr(foo(bar)))
+#' as_label(expr(foobar))
+#'
+#' # It works with any R object. This is also useful for quoted
+#' # arguments because the user might unquote constant objects:
+#' as_label(1:3)
+#' as_label(base::list)
+#' @export
+as_label <- function(x) {
+  x <- quo_squash(x)
+
+  if (is_missing(x)) {
+    return("<empty>")
+  }
+
+  switch(typeof(x),
+    NULL = "NULL",
+    symbol = as_string(x),
+    language = {
+      if (is_data_pronoun(x)) {
+        data_pronoun_name(x) %||% "<unknown>"
+      } else {
+        name <- deparse_one(x)
+        name <- gsub("\n.*$", "...", name)
+        name
+      }
+    },
+    if (is_bare_atomic(x, n = 1)) {
+      name <- expr_text(x)
+      name <- gsub("\n.*$", "...", name)
+      name
+    } else {
+      paste0("<", rlang_type_sum(x), ">")
+    }
+  )
+}
+
+#' Extract names from symbols
+#'
+#' @description
+#'
+#' `as_name()` converts [symbols][sym] to character strings. The
+#' conversion is deterministic. That is, the roundtrip symbol -> name
+#' -> symbol always gets the same result.
+#'
+#' - Use `as_name()` when you need to transform a symbol to a string
+#'   to _refer_ to an object by its name.
+#'
+#' - Use [as_label()] when you need to transform any kind of object to
+#'   a string to _represent_ that object with a short description.
+#'
+#' Expect `as_name()` to gain
+#' [name-repairing](https://principles.tidyverse.org/names-attribute.html#minimal-unique-universal)
+#' features in the future.
+#'
+#' Note that `rlang::as_name()` is the _opposite_ of
+#' [base::as.name()]. If you're writing base R code, we recommend
+#' using [base::as.symbol()] which is an alias of `as.name()` that
+#' follows a more modern terminology (R types instead of S modes).
+#'
+#' @param x A string or symbol, possibly wrapped in a [quosure][quosure].
+#'   If a string, the attributes are removed, if any.
+#' @return A character vector of length 1.
+#'
+#' @seealso [as_label()] for converting any object to a single string
+#'   suitable as a label. [as_string()] for a lower-level version that
+#'   doesn't unwrap quosures.
+#' @examples
+#' # Let's create some symbols:
+#' foo <- quote(foo)
+#' bar <- sym("bar")
+#'
+#' # as_name() converts symbols to strings:
+#' foo
+#' as_name(foo)
+#'
+#' typeof(bar)
+#' typeof(as_name(bar))
+#'
+#' # as_name() unwraps quosured symbols automatically:
+#' as_name(quo(foo))
+#' @export
+as_name <- function(x) {
+  if (is_quosure(x)) {
+    x <- quo_get_expr(x)
+  }
+  as_string(x)
+}
