@@ -9,25 +9,6 @@ test_that("names2() fails for environments", {
   expect_error(names2(env()), "Use env_names() for environments.", fixed = TRUE)
 })
 
-test_that("set_attrs() fails with uncopyable types", {
-  expect_error(set_attrs(env(), foo = "bar"), "is uncopyable")
-})
-
-test_that("mut_attrs() fails with copyable types", {
-  expect_error(mut_attrs(letters, foo = "bar"), "is copyable")
-})
-
-test_that("set_attrs() called with NULL zaps attributes", {
-  obj <- set_attrs(letters, foo = "bar")
-  expect_identical(set_attrs(obj, NULL), letters)
-})
-
-test_that("set_attrs() does not zap old attributes", {
-  obj <- set_attrs(letters, foo = "bar")
-  obj <- set_attrs(obj, baz = "bam")
-  expect_named(attributes(obj), c("foo", "baz"))
-})
-
 test_that("inputs must be valid", {
   expect_error(set_names(environment()), "must be a vector")
   expect_error(set_names(1:10, letters[1:4]), "same length")
@@ -54,3 +35,7 @@ test_that("set_names() coerces to character", {
   expect_identical(set_names(1L, TRUE), c(`TRUE` = 1L))
   expect_identical(set_names(1:2, "a", TRUE), c(a = 1L, `TRUE` = 2L))
 })
+
+test_that("has_name() works with pairlists", {
+  expect_true(has_name(fn_fmls(`[.data.frame`), "drop"))
+})
